import { VirtualElement, ComputePositionReturn, ComputePositionConfig, AutoUpdateOptions } from '@floating-ui/dom';
export * from '@floating-ui/dom';
import { Ref, ComputedRef } from 'vue-demi';
import { MaybeRef as MaybeRef$1 } from 'vue-reactivity-fantasy';

type MaybeRef<T> = T | Ref<T>;
type Nullable = null | undefined;
type ReferenceType = Element | VirtualElement;
type FloatingType = HTMLElement;
type UseFloatingData = Omit<ComputePositionReturn, 'x' | 'y'> & {
    x: number | null;
    y: number | null;
};

interface UseFloatingOptions extends Omit<Partial<ComputePositionConfig>, 'platform'> {
    /**
     * @default false
     */
    disabled?: boolean;
}
interface UseFloatingReturn {
    data: ComputedRef<UseFloatingData>;
    update: () => void;
    stop: () => void;
}
/**
 * Uue hooks based on `computePosition`.
 * @see https://floating-ui.com/docs/computePosition
 */
declare function useFloating(reference: MaybeRef$1<ReferenceType | Nullable>, floating: MaybeRef$1<FloatingType | Nullable>, userOptions?: MaybeRef$1<UseFloatingOptions>): UseFloatingReturn;

interface UseAutoUpdateOptions extends Partial<AutoUpdateOptions> {
    /**
     * @default false
     */
    disabled?: boolean;
}
/**
 * Automatically updates the position of the floating element when necessary.
 * @see https://floating-ui.com/docs/autoUpdate
 */
declare function useAutoUpdate(reference: MaybeRef<ReferenceType | Nullable>, floating: MaybeRef<FloatingType | Nullable>, update: () => void, options?: MaybeRef<UseAutoUpdateOptions>): () => void;

export { FloatingType, MaybeRef, Nullable, ReferenceType, UseAutoUpdateOptions, UseFloatingData, UseFloatingOptions, UseFloatingReturn, useAutoUpdate, useFloating };
