import { computePosition, autoUpdate } from '@floating-ui/dom';
export * from '@floating-ui/dom';
import { ref, unref, computed } from 'vue-demi';
import { useTransformState, createManualEffect, useEffect } from 'vue-reactivity-fantasy';

function deepEqual(a, b) {
  if (a === b) {
    return true;
  }
  if (typeof a !== typeof b) {
    return false;
  }
  if (typeof a === "function" && a.toString() === b.toString()) {
    return true;
  }
  let length, i, keys;
  if (a && b && typeof a == "object") {
    if (Array.isArray(a)) {
      length = a.length;
      if (length != b.length)
        return false;
      for (i = length; i-- !== 0; ) {
        if (!deepEqual(a[i], b[i])) {
          return false;
        }
      }
      return true;
    }
    keys = Object.keys(a);
    length = keys.length;
    if (length !== Object.keys(b).length) {
      return false;
    }
    for (i = length; i-- !== 0; ) {
      if (!Object.prototype.hasOwnProperty.call(b, keys[i])) {
        return false;
      }
    }
    for (i = length; i-- !== 0; ) {
      const key = keys[i];
      if (!deepEqual(a[key], b[key])) {
        return false;
      }
    }
    return true;
  }
  return a !== a && b !== b;
}

const defaultOptions = {
  strategy: "absolute",
  placement: "bottom"
};
function useFloating(reference, floating, userOptions = {}) {
  const optionsRef = useTransformState(userOptions, (userOptions2) => {
    return {
      ...defaultOptions,
      ...userOptions2,
      middleware: userOptions2.middleware && [...userOptions2.middleware]
    };
  });
  const computePositionReturnRef = ref();
  const floatingDataRef = useTransformState(
    computePositionReturnRef,
    (computedPositionReturn) => {
      const { value: options } = optionsRef;
      return {
        x: null,
        y: null,
        middlewareData: {},
        strategy: options.strategy,
        placement: options.placement,
        ...computedPositionReturn
      };
    },
    false
  );
  const update = () => {
    const referenceEl = unref(reference);
    const floatingEl = unref(floating);
    if (referenceEl && floatingEl) {
      computePosition(referenceEl, floatingEl, optionsRef.value).then((returnData) => {
        if (!deepEqual(computePositionReturnRef.value, returnData)) {
          computePositionReturnRef.value = returnData;
        }
      });
    }
  };
  const updateEffectControl = createManualEffect(() => {
    return useEffect(
      () => {
        update();
      },
      [() => unref(reference), () => unref(floating)],
      {
        immediate: true
      }
    );
  });
  const cleanupOptionsEffect = useEffect(
    (_, options, lastOptions) => {
      if (deepEqual(options, lastOptions)) {
        return;
      }
      if (!options.disabled) {
        updateEffectControl.reset();
      } else {
        updateEffectControl.clear();
      }
    },
    optionsRef,
    {
      immediate: true,
      deep: true
    }
  );
  return {
    data: floatingDataRef,
    update,
    stop: () => {
      cleanupOptionsEffect();
      updateEffectControl.clear();
    }
  };
}

function useAutoUpdate(reference, floating, update, options = {}) {
  const optionsRef = computed(() => unref(options));
  const autoUpdateEffectControl = createManualEffect(() => {
    return useEffect(
      (onCleanup, [reference2, floating2]) => {
        if (reference2 && floating2) {
          const stopAutoUpdate = autoUpdate(reference2, floating2, update, optionsRef.value);
          onCleanup(stopAutoUpdate);
        }
      },
      [() => unref(reference), () => unref(floating)],
      {
        immediate: true
      }
    );
  });
  const cleanupOptionsEffect = useEffect(
    (_, options2) => {
      if (!options2.disabled) {
        autoUpdateEffectControl.reset();
      } else {
        autoUpdateEffectControl.clear();
      }
    },
    optionsRef,
    {
      immediate: true,
      deep: true
    }
  );
  return () => {
    cleanupOptionsEffect();
    autoUpdateEffectControl.clear();
  };
}

export { useAutoUpdate, useFloating };
